@@ -163,7 +163,7 @@ public:
 
     /** order pattern by number of character states and return in ptn_order
     */
-    void orderPatternByNumChars();
+    virtual void orderPatternByNumChars();
 
     /**
      * un-group site-patterns, i.e., making #sites = #patterns and pattern frequency = 1 for all patterns
@@ -579,6 +579,11 @@ public:
      */
     virtual double computeUnconstrainedLogL();
 
+    /**
+     * 	@return number of states, if it is a partition model, return max num_states across all partitions
+     */
+    virtual int getMaxNumStates() { return num_states; }
+
     /** either SEQ_BINARY, SEQ_DNA, SEQ_PROTEIN, SEQ_MORPH, or SEQ_CODON */
     SeqType seq_type;
 
@@ -593,6 +598,11 @@ public:
             fraction of constant sites
      */
     double frac_const_sites;
+    
+    /**
+            fraction of invariant sites, incl. const sites and site like G-S-GG-GGGG
+     */
+    double frac_invariant_sites;
 
     /** number of informative sites */
     int num_informative_sites;
@@ -616,6 +626,14 @@ public:
 
     vector<vector<int> > seq_states; // state set for each sequence in the alignment
 
+    /* for site-specific state frequency model with Huaichun, Edward, Andrew */
+    
+    /* site to model ID map */
+    IntVector site_model;
+    
+    /** site to state frequency vector */
+    vector<double*> site_state_freq;
+
     /**
      * @return true if data type is SEQ_CODON and state is a stop codon
      */
@@ -672,6 +690,16 @@ public:
 
     void getAppearance(char state, StateBitset &state_app);
 
+	/**
+	 * read site specific state frequency vectors from a file to create corresponding model
+     * update site_model and site_state_freq variables for this class
+	 * @param aln input alignment
+	 * @param site_freq_file file name
+     * @return TRUE if alignment needs to be changed, FALSE otherwise
+	 */
+	bool readSiteStateFreq(const char* site_freq_file);
+
+
 protected:
 
 
