@@ -10,12 +10,16 @@
 #include "timeutil.h"
 #include "gzstream.h"
 
+const char* CKP_HEADER = "--- # IQ-TREE Checkpoint";
+
 
 Checkpoint::Checkpoint() {
 	filename = "";
     prev_dump_time = 0;
     dump_interval = 30; // dumping at most once per 30 seconds
     struct_name = "";
+    compression = true;
+    header = CKP_HEADER;
 }
 
 
@@ -23,11 +27,50 @@ Checkpoint::~Checkpoint() {
 }
 
 
-const char* CKP_HEADER = "--- # IQ-TREE Checkpoint";
-
 void Checkpoint::setFileName(string filename) {
 	this->filename = filename;
 }
+
+
+void Checkpoint::load(istream &in) {
+    string line;
+    string struct_name;
+    size_t pos;
+    int listid = 0;
+    while (!in.eof()) {
+        getline(in, line);
+        pos = line.find('#');
+        if (pos != string::npos)
+            line.erase(pos);
+        line.erase(line.find_last_not_of("\n\r\t")+1);
+//            trimString(line);
+        if (line.empty()) continue;
+        if (line[0] != ' ') {
+            struct_name = "";
+        }
+//            trimString(line);
+        line.erase(0, line.find_first_not_of(" \n\r\t"));
+        if (line.empty()) continue;
+        pos = line.find(": ");
+        if (pos != string::npos) {
+            // mapping
+            (*this)[struct_name + line.substr(0, pos)] = line.substr(pos+2);
+        } else if (line[line.length()-1] == ':') {
+            // start a new struct
+            line.erase(line.length()-1);
+            trimString(line);
+            struct_name = line + '.';
+            listid = 0;
+            continue;
+        } else {
+            // collection
+            (*this)[struct_name + convertIntToString(listid)] = line;
+            listid++;
+        }
+    }
+}
+
+
 void Checkpoint::load() {
 	assert(filename != "");
     if (!fileExists(filename)) return;
@@ -43,43 +86,10 @@ void Checkpoint::load() {
             in.close();
             return;
         }
-        if (line != CKP_HEADER)
+        if (line != header)
         	throw ("Invalid checkpoint file " + filename);
-        string struct_name;
-        size_t pos;
-        int listid = 0;
-        while (!in.eof()) {
-        	getline(in, line);
-            pos = line.find('#');
-            if (pos != string::npos)
-                line.erase(pos);
-            line.erase(line.find_last_not_of("\n\r\t")+1);
-//            trimString(line);
-            if (line.empty()) continue;
-            if (line[0] != ' ') {
-                struct_name = "";
-            }
-//            trimString(line);
-            line.erase(0, line.find_first_not_of(" \n\r\t"));
-            if (line.empty()) continue;
-        	pos = line.find(": ");
-        	if (pos != string::npos) {
-                // mapping
-                (*this)[struct_name + line.substr(0, pos)] = line.substr(pos+2);
-            } else if (line[line.length()-1] == ':') {
-                // start a new struct
-                line.erase(line.length()-1);
-                trimString(line);
-                struct_name = line + '.';
-                listid = 0;
-                continue;
-            } else {
-                // collection
-                (*this)[struct_name + convertIntToString(listid)] = line;
-                listid++;
-//        		throw "':' is expected between key and value";
-            }
-        }
+        // call load from the stream
+        load(in);
         in.clear();
         // set the failbit again
         in.exceptions(ios::failbit | ios::badbit);
@@ -93,10 +103,39 @@ void Checkpoint::load() {
     }
 }
 
+void Checkpoint::setCompression(bool compression) {
+    this->compression = compression;
+}
+
+/**
+    set the header line to overwrite the default header
+    @param header header line
+*/
+void Checkpoint::setHeader(string header) {
+    this->header = "--- # " + header;
+}
+
 void Checkpoint::setDumpInterval(double interval) {
     dump_interval = interval;
 }
 
+void Checkpoint::dump(ostream &out) {
+    string struct_name;
+    size_t pos;
+    int listid = 0;
+    for (iterator i = begin(); i != end(); i++) {
+        if ((pos = i->first.find('.')) != string::npos) {
+            if (struct_name != i->first.substr(0, pos)) {
+                struct_name = i->first.substr(0, pos);
+                out << struct_name << ':' << endl;
+                listid = 0;
+            }
+            // check if key is a collection
+            out << ' ' << i->first.substr(pos+1) << ": " << i->second << endl;
+        } else
+            out << i->first << ": " << i->second << endl;
+    }
+}
 
 void Checkpoint::dump(bool force) {
     if (filename == "")
@@ -107,26 +146,20 @@ void Checkpoint::dump(bool force) {
     }
     prev_dump_time = getRealTime();
     try {
-        ogzstream out;
-        out.exceptions(ios::failbit | ios::badbit);
-        out.open(filename.c_str());
-        out << CKP_HEADER << endl;
-        string struct_name;
-        size_t pos;
-        int listid = 0;
-        for (iterator i = begin(); i != end(); i++) {
-            if ((pos = i->first.find('.')) != string::npos) {
-                if (struct_name != i->first.substr(0, pos)) {
-                    struct_name = i->first.substr(0, pos);
-                    out << struct_name << ":" << endl;
-                    listid = 0;
-                }
-                // check if key is a collection
-                out << "  " << i->first.substr(pos+1) << ": " << i->second << endl;
-            } else
-                out << i->first << ": " << i->second << endl;
-        }
-        out.close();
+        ostream *out;
+        if (compression) 
+            out = new ogzstream(filename.c_str());
+        else
+            out = new ofstream(filename.c_str()); 
+        out->exceptions(ios::failbit | ios::badbit);
+        *out << header << endl;
+        // call dump stream
+        dump(*out);
+        if (compression)
+            ((ogzstream*)out)->close();
+        else
+            ((ofstream*)out)->close();
+        delete out;
 //        cout << "Checkpoint dumped" << endl;
     } catch (ios::failure &) {
         outError(ERR_WRITE_OUTPUT, filename.c_str());
@@ -198,6 +231,13 @@ void Checkpoint::startList(int nelem) {
         list_element_precision.push_back(0);
 }
 
+void Checkpoint::setListElement(int id) {
+    list_element.back() = id;
+    stringstream ss;
+    ss << setw(list_element_precision.back()) << setfill('0') << list_element.back();
+    struct_name += ss.str() + ".";
+}
+
 void Checkpoint::addListElement() {
     list_element.back()++;
     if (list_element.back() > 0) {
