@@ -110,12 +110,12 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
 
 	if (model_str == "") {
 		if (tree->aln->seq_type == SEQ_DNA) model_str = "HKY";
-		else if (tree->aln->seq_type == SEQ_PROTEIN) model_str = "WAG";
+		else if (tree->aln->seq_type == SEQ_PROTEIN) model_str = "LG";
 		else if (tree->aln->seq_type == SEQ_BINARY) model_str = "GTR2";
 		else if (tree->aln->seq_type == SEQ_CODON) model_str = "GY";
 		else if (tree->aln->seq_type == SEQ_MORPH) model_str = "MK";
 		else model_str = "JC";
-		outWarning("Default model may be under-fitting. Use option '-m TEST' to select best-fit model.");
+		outWarning("Default model "+model_str + " may be under-fitting. Use option '-m TEST' to determine the best-fit model.");
 	}
 
 	/********* preprocessing model string ****************/
@@ -210,7 +210,13 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
     }
 
     // then normal frequency
-	posfreq = rate_str.find("+F");
+    if (rate_str.find("+FO") != string::npos)
+        posfreq = rate_str.find("+FO");
+    else if (rate_str.find("+Fo") != string::npos)
+        posfreq = rate_str.find("+Fo");
+    else
+        posfreq = rate_str.find("+F");
+        
     bool optimize_mixmodel_weight = params.optimize_mixmodel_weight;
 
 	if (posfreq != string::npos) {
@@ -251,9 +257,10 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
             else
                 freq_type = FREQ_EQUAL;
 		} else if (freq_str == "+FO" || freq_str == "+Fo") {
-            if (freq_type == FREQ_MIXTURE)
-                outError("Mixture frequency with optimized frequency is not allowed");
-            else
+            if (freq_type == FREQ_MIXTURE) {
+                freq_params = "optimize," + freq_params;
+                optimize_mixmodel_weight = true;                
+            } else
                 freq_type = FREQ_ESTIMATE;
 		} else if (freq_str == "+F1x4" || freq_str == "+F1X4") {
             if (freq_type == FREQ_MIXTURE)
@@ -276,7 +283,7 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
 
 	/******************** initialize model ****************************/
 
-	if (!params.site_freq_file) {
+	if (tree->aln->site_state_freq.empty()) {
 		if (model_str.substr(0, 3) == "MIX" || freq_type == FREQ_MIXTURE) {
 			string model_list;
 			if (model_str.substr(0, 3) == "MIX") {
@@ -303,24 +310,15 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
 		model = new ModelSet(model_str.c_str(), tree);
 		ModelSet *models = (ModelSet*)model; // assign pointer for convenience
 		models->init((params.freq_type != FREQ_UNKNOWN) ? params.freq_type : FREQ_EMPIRICAL);
-		IntVector site_model;
-		vector<double*> freq_vec;
-		bool aln_changed = readSiteFreq(tree->aln, params.site_freq_file, site_model, freq_vec);
-        if (aln_changed) {
-            cout << "Regrouping alignment sites..." << endl;
-            tree->aln->regroupSitePattern(freq_vec.size(), site_model);
-            //tree->aln->ungroupSitePattern();
-            tree->setAlignment(tree->aln);
-        }
 		int i;
 		models->pattern_model_map.resize(tree->aln->getNPattern(), -1);
 		for (i = 0; i < tree->aln->getNSite(); i++) {
-			models->pattern_model_map[tree->aln->getPatternID(i)] = site_model[i];
+			models->pattern_model_map[tree->aln->getPatternID(i)] = tree->aln->site_model[i];
 			//cout << "site " << i << " ptn " << tree->aln->getPatternID(i) << " -> model " << site_model[i] << endl;
 		}
 		double *state_freq = new double[model->num_states];
 		double *rates = new double[model->getNumRateEntries()];
-		for (i = 0; i < freq_vec.size(); i++) {
+		for (i = 0; i < tree->aln->site_state_freq.size(); i++) {
 			ModelGTR *modeli;
 			if (i == 0) {
 				modeli = (ModelGTR*)createModel(model_str, models_block, (params.freq_type != FREQ_UNKNOWN) ? params.freq_type : FREQ_EMPIRICAL, "", tree, true);
@@ -331,21 +329,20 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
 				modeli->setStateFrequency(state_freq);
 				modeli->setRateMatrix(rates);
 			}
-			if (freq_vec[i])
-				modeli->setStateFrequency (freq_vec[i]);
+			if (tree->aln->site_state_freq[i])
+				modeli->setStateFrequency (tree->aln->site_state_freq[i]);
 
 			modeli->init(FREQ_USER_DEFINED);
 			models->push_back(modeli);
 		}
 		delete [] rates;
 		delete [] state_freq;
-		cout << "Alignment is divided into " << models->size() << " partitions with " << tree->aln->getNPattern() << " patterns" << endl;
-		for (vector<double*>::reverse_iterator it = freq_vec.rbegin(); it != freq_vec.rend(); it++)
-			if (*it) delete [] (*it);
-            
+
+        models->joinEigenMemory();
+
         // delete information of the old alignment
-        tree->aln->ordered_pattern.clear();
-        tree->deleteAllPartialLh();
+//        tree->aln->ordered_pattern.clear();
+//        tree->deleteAllPartialLh();
 	}
     
 //	if (model->isMixture())
@@ -362,8 +359,23 @@ ModelFactory::ModelFactory(Params &params, PhyloTree *tree, ModelsBlock *models_
 		tree->aln->buildSeqStates(true);
 //		if (unobserved_ptns.size() <= 0)
 //			outError("Invalid use of +ASC because all constant patterns are observed in the alignment");
-		if (unobserved_ptns.size() < tree->aln->getNumNonstopCodons())
-			outError("Invalid use of +ASC because constant patterns are observed in the alignment");
+		if (tree->aln->frac_invariant_sites > 0) {
+//            cerr << tree->aln->frac_invariant_sites*tree->aln->getNSite() << " invariant sites observed in the alignment" << endl;
+//            for (Alignment::iterator pit = tree->aln->begin(); pit != tree->aln->end(); pit++)
+//                if (pit->isInvariant()) {
+//                    string pat_str = "";
+//                    for (Pattern::iterator it = pit->begin(); it != pit->end(); it++)
+//                        pat_str += tree->aln->convertStateBackStr(*it);
+//                    cerr << pat_str << " is invariant site pattern" << endl;
+//                }
+            if (!params.partition_file) {                
+                string varsites_file = ((string)params.out_prefix + ".varsites.phy");
+                tree->aln->printPhylip(varsites_file.c_str(), false, NULL, false, true);
+                cerr << "For your convenience alignment with variable sites printed to " << varsites_file << endl;
+            } 
+            outError("Invalid use of +ASC because of " + convertIntToString(tree->aln->frac_invariant_sites*tree->aln->getNSite()) +
+                " invariant sites in the alignment");
+        }
 		cout << "Ascertainment bias correction: " << unobserved_ptns.size() << " unobservable constant patterns"<< endl;
 		rate_str = rate_str.substr(0, posasc) + rate_str.substr(posasc+4);
 	}
@@ -593,102 +605,7 @@ int ModelFactory::getNParameters() {
 	int df = model->getNDim() + model->getNDimFreq() + site_rate->getNDim() + site_rate->phylo_tree->branchNum;
 	return df;
 }
-bool ModelFactory::readSiteFreq(Alignment *aln, char* site_freq_file, IntVector &site_model, vector<double*> &freq_vec)
-{
-	cout << "Reading site-specific state frequency file " << site_freq_file << " ..." << endl;
-	site_model.resize(aln->getNSite(), -1);
-    int i;
-    IntVector pattern_to_site; // vector from pattern to the first site
-    pattern_to_site.resize(aln->getNPattern(), -1);
-    for (i = 0; i < aln->getNSite(); i++)
-        if (pattern_to_site[aln->getPatternID(i)] == -1)
-            pattern_to_site[aln->getPatternID(i)] = i;
-            
-    bool aln_changed = false;
-    
-	try {
-		ifstream in;
-		in.exceptions(ios::failbit | ios::badbit);
-		in.open(site_freq_file);
-		double freq;
-		string site_spec;
-		int specified_sites = 0;
-		in.exceptions(ios::badbit);
-		for (int model_id = 0; !in.eof(); model_id++) {
-			// remove the failbit
-			in >> site_spec;
-			if (in.eof()) break;
-			IntVector site_id;
-			extractSiteID(aln, site_spec.c_str(), site_id);
-			specified_sites += site_id.size();
-			if (site_id.size() == 0) throw "No site ID specified";
-			for (IntVector::iterator it = site_id.begin(); it != site_id.end(); it++) {
-				if (site_model[*it] != -1) throw "Duplicated site ID";
-				site_model[*it] = freq_vec.size();
-			}
-			double *site_freq_entry = new double[aln->num_states];
-			double sum = 0;
-			for (i = 0; i < aln->num_states; i++) {
-				in >> freq;
-				if (freq <= 0.0 || freq >= 1.0) throw "Invalid frequency entry";
-				site_freq_entry[i] = freq;
-				sum += freq;
-			}
-			if (fabs(sum-1.0) > 1e-4) {
-                if (fabs(sum-1.0) > 1e-3)
-                    outWarning("Frequencies of site " + site_spec + " do not sum up to 1 and will be normalized");
-                sum = 1.0/sum;
-                for (i = 0; i < aln->num_states; i++) 
-                    site_freq_entry[i] *= sum;
-            }
-			aln->convfreq(site_freq_entry); // regularize frequencies (eg if some freq = 0)
-            
-            // 2016-02-01: now check for equality of sites with same site-pattern and same freq
-            int prev_site = pattern_to_site[aln->getPatternID(site_id[0])];
-            if (site_id.size() == 1 && prev_site < site_id[0] && site_model[prev_site] != -1) {
-                // compare freq with prev_site
-                bool matched_freq = true;
-                double *prev_freq = freq_vec[site_model[prev_site]];
-                for (i = 0; i < aln->num_states; i++) {
-                    if (site_freq_entry[i] != prev_freq[i]) {
-                        matched_freq = false;
-                        break;
-                    }
-                }
-                if (matched_freq) {
-                    site_model[site_id[0]] = site_model[prev_site];
-                } else
-                    aln_changed = true;
-            }
-            
-            if (site_model[site_id[0]] == freq_vec.size())
-                freq_vec.push_back(site_freq_entry);
-            else
-                delete [] site_freq_entry;
-		}
-		if (specified_sites < site_model.size()) {
-            aln_changed = true;
-			// there are some unspecified sites
-			cout << site_model.size() - specified_sites << " unspecified sites will get default frequencies" << endl;
-			for (i = 0; i < site_model.size(); i++)
-				if (site_model[i] == -1) 
-					site_model[i] = freq_vec.size();
-			freq_vec.push_back(NULL);
-		}
-		in.clear();
-		// set the failbit again
-		in.exceptions(ios::failbit | ios::badbit);
-		in.close();
-	} catch (const char* str) {
-		outError(str);
-	} catch (string str) {
-		outError(str);
-	} catch(ios::failure) {
-		outError(ERR_READ_INPUT);
-	}
-    return aln_changed;
-}
-
+/*
 double ModelFactory::initGTRGammaIParameters(RateHeterogeneity *rate, ModelSubst *model, double initAlpha,
                                            double initPInvar, double *initRates, double *initStateFreqs)  {
 
@@ -702,7 +619,7 @@ double ModelFactory::initGTRGammaIParameters(RateHeterogeneity *rate, ModelSubst
     site_rate->phylo_tree->clearAllPartialLH();
     return site_rate->phylo_tree->computeLikelihood();
 }
-
+*/
 double ModelFactory::optimizeParametersOnly(double gradient_epsilon) {
 	double logl;
 	/* Optimize substitution and heterogeneity rates independently */
@@ -768,8 +685,8 @@ double ModelFactory::optimizeAllParameters(double gradient_epsilon) {
     return score;
 }
 
-double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_info, double logl_epsilon, double gradient_epsilon) {
-    if (!site_rate->isGammai())
+double ModelFactory::optimizeParametersGammaInvar(int fixed_len, bool write_info, double logl_epsilon, double gradient_epsilon) {
+    if (!site_rate->isGammai() || site_rate->isFixPInvar() || site_rate->isFixGammaShape() || site_rate->getTree()->aln->frac_const_sites == 0.0)
         return optimizeParameters(fixed_len, write_info, logl_epsilon, gradient_epsilon);
         
 	double begin_time = getRealTime();
@@ -783,16 +700,20 @@ double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_inf
 	DoubleVector bestLens;
 	tree->saveBranchLengths(initBranLens);
     bestLens = initBranLens;
-	int numRateEntries = tree->getModel()->getNumRateEntries();
-	double *rates = new double[numRateEntries];
-	double *bestRates = new double[numRateEntries];
-	tree->getModel()->getRateMatrix(rates);
-	int numStates = tree->aln->num_states;
-	double *state_freqs = new double[numStates];
-	tree->getModel()->getStateFrequency(state_freqs);
+//	int numRateEntries = tree->getModel()->getNumRateEntries();
+    Checkpoint *model_ckp = new Checkpoint;
+    Checkpoint *best_ckp = new Checkpoint;
+    Checkpoint *saved_ckp = model->getCheckpoint();
+    *model_ckp = *saved_ckp;
+//	double *rates = new double[numRateEntries];
+//	double *bestRates = new double[numRateEntries];
+//	tree->getModel()->getRateMatrix(rates);
+//	int numStates = tree->aln->num_states;
+//	double *state_freqs = new double[numStates];
+//	tree->getModel()->getStateFrequency(state_freqs);
 
 	/* Best estimates found */
-	double *bestStateFreqs =  new double[numStates];
+//	double *bestStateFreqs =  new double[numStates];
 	double bestLogl = -DBL_MAX;
 	double bestAlpha = 0.0;
 	double bestPInvar = 0.0;
@@ -810,8 +731,8 @@ double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_inf
                 cout << "Testing with init. pinv = " << initPInv << " / init. alpha = "  << initAlpha << endl;
             }
 
-            vector<double> estResults = optimizeGammaInvWithInitValue(fixed_len, logl_epsilon, gradient_epsilon, tree, site_rate, rates, state_freqs,
-                                                                   initPInv, initAlpha, initBranLens);
+            vector<double> estResults = optimizeGammaInvWithInitValue(fixed_len, logl_epsilon, gradient_epsilon,
+                                                                   initPInv, initAlpha, initBranLens, model_ckp);
 
 
             if (write_info) {
@@ -825,8 +746,13 @@ double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_inf
                 bestPInvar = estResults[0];
                 bestLens.clear();
                 tree->saveBranchLengths(bestLens);
-                tree->getModel()->getRateMatrix(bestRates);
-                tree->getModel()->getStateFrequency(bestStateFreqs);
+                model->setCheckpoint(best_ckp);
+                model->saveCheckpoint();
+                model->setCheckpoint(saved_ckp);
+//                *best_ckp = *saved_ckp;
+
+//                tree->getModel()->getRateMatrix(bestRates);
+//                tree->getModel()->getStateFrequency(bestStateFreqs);
                 if (estResults[0] < initPInv) {
                     initPInv = estResults[0] - testInterval;
                     if (initPInv < 0.0)
@@ -850,11 +776,11 @@ double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_inf
             }
             vector<double> estResults; // vector of p_inv, alpha and logl
             if (Params::getInstance().opt_gammai_keep_bran)
-                estResults = optimizeGammaInvWithInitValue(fixed_len, logl_epsilon, gradient_epsilon, tree, site_rate, rates, state_freqs,
-                                                                          initPInv, initAlpha, bestLens);
+                estResults = optimizeGammaInvWithInitValue(fixed_len, logl_epsilon, gradient_epsilon,
+                                                                          initPInv, initAlpha, bestLens, model_ckp);
             else
-                estResults = optimizeGammaInvWithInitValue(fixed_len, logl_epsilon, gradient_epsilon, tree, site_rate, rates, state_freqs,
-                                                                      initPInv, initAlpha, initBranLens);
+                estResults = optimizeGammaInvWithInitValue(fixed_len, logl_epsilon, gradient_epsilon,
+                                                                      initPInv, initAlpha, initBranLens, model_ckp);
             if (write_info) {
                 cout << "Est. p_inv: " << estResults[0] << " / Est. gamma shape: " << estResults[1]
                 << " / Logl: " << estResults[2] << endl;
@@ -862,39 +788,50 @@ double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_inf
 
             initPInv = initPInv + testInterval;
 
-            if (estResults[2] > bestLogl + logl_epsilon) {
+            if (estResults[2] > bestLogl) {
                 bestLogl = estResults[2];
                 bestAlpha = estResults[1];
                 bestPInvar = estResults[0];
                 bestLens.clear();
                 tree->saveBranchLengths(bestLens);
-                tree->getModel()->getRateMatrix(bestRates);
-                tree->getModel()->getStateFrequency(bestStateFreqs);
+                model->setCheckpoint(best_ckp);
+                model->saveCheckpoint();
+                model->setCheckpoint(saved_ckp);
+//                *best_ckp = *saved_ckp;
+
+//                tree->getModel()->getRateMatrix(bestRates);
+//                tree->getModel()->getStateFrequency(bestStateFreqs);
             }
         }
     }
 
     site_rate->setGammaShape(bestAlpha);
     site_rate->setPInvar(bestPInvar);
-	((ModelGTR*) tree->getModel())->setRateMatrix(bestRates);
-	((ModelGTR*) tree->getModel())->setStateFrequency(bestStateFreqs);
+    model->setCheckpoint(best_ckp);
+    model->restoreCheckpoint();
+    model->setCheckpoint(saved_ckp);
+//	((ModelGTR*) tree->getModel())->setRateMatrix(bestRates);
+//	((ModelGTR*) tree->getModel())->setStateFrequency(bestStateFreqs);
 	tree->restoreBranchLengths(bestLens);
-	tree->getModel()->decomposeRateMatrix();
+//	tree->getModel()->decomposeRateMatrix();
 
 	tree->clearAllPartialLH();
 	tree->setCurScore(tree->computeLikelihood());
-    assert(fabs(tree->getCurScore() - bestLogl) < 1.0);
-    if (write_info) {    
+    if (write_info) {
         cout << endl;
         cout << "Best p_inv: " << bestPInvar << " / best gamma shape: " << bestAlpha << " / ";
         cout << "Logl: " << tree->getCurScore() << endl;
     }
+    assert(fabs(tree->getCurScore() - bestLogl) < 1.0);
+
+//	delete [] rates;
+//	delete [] state_freqs;
+//	delete [] bestRates;
+//	delete [] bestStateFreqs;
+
+    delete model_ckp;
+    delete best_ckp;
 
-	delete [] rates;
-	delete [] state_freqs;
-	delete [] bestRates;
-	delete [] bestStateFreqs;
-    
 	double elapsed_secs = getRealTime() - begin_time;
 	if (write_info)
 		cout << "Parameters optimization took " << elapsed_secs << " sec" << endl;
@@ -906,22 +843,26 @@ double ModelFactory::optimizeParametersGammaInvar(bool fixed_len, bool write_inf
     return tree->getCurScore();
 }
 
-vector<double> ModelFactory::optimizeGammaInvWithInitValue(bool fixed_len, double logl_epsilon, double gradient_epsilon,
-                                                 PhyloTree *tree, RateHeterogeneity *site_rates, double *rates,
-                                                 double *state_freqs, double initPInv, double initAlpha,
-                                                 DoubleVector &lenvec) {
+vector<double> ModelFactory::optimizeGammaInvWithInitValue(int fixed_len, double logl_epsilon, double gradient_epsilon,
+                                                 double initPInv, double initAlpha,
+                                                 DoubleVector &lenvec, Checkpoint *model_ckp) {
+    PhyloTree *tree = site_rate->getTree();
     tree->restoreBranchLengths(lenvec);
-    ((ModelGTR*) tree->getModel())->setRateMatrix(rates);
-    ((ModelGTR*) tree->getModel())->setStateFrequency(state_freqs);
-    tree->getModel()->decomposeRateMatrix();
-    site_rates->setPInvar(initPInv);
-    site_rates->setGammaShape(initAlpha);
+    Checkpoint *saved_ckp = model->getCheckpoint();
+    model->setCheckpoint(model_ckp);
+    model->restoreCheckpoint();
+    model->setCheckpoint(saved_ckp);
+//    ((ModelGTR*) tree->getModel())->setRateMatrix(rates);
+//    ((ModelGTR*) tree->getModel())->setStateFrequency(state_freqs);
+//    tree->getModel()->decomposeRateMatrix();
+    site_rate->setPInvar(initPInv);
+    site_rate->setGammaShape(initAlpha);
     tree->clearAllPartialLH();
     optimizeParameters(fixed_len, false, logl_epsilon, gradient_epsilon);
 
     vector<double> estResults;
-    double estPInv = tree->getRate()->getPInvar();
-    double estAlpha = tree->getRate()->getGammaShape();
+    double estPInv = site_rate->getPInvar();
+    double estAlpha = site_rate->getGammaShape();
     double logl = tree->getCurScore();
     estResults.push_back(estPInv);
     estResults.push_back(estAlpha);
@@ -930,12 +871,12 @@ vector<double> ModelFactory::optimizeGammaInvWithInitValue(bool fixed_len, doubl
 }
 
 
-double ModelFactory::optimizeParameters(bool fixed_len, bool write_info,
+double ModelFactory::optimizeParameters(int fixed_len, bool write_info,
                                         double logl_epsilon, double gradient_epsilon) {
 	assert(model);
 	assert(site_rate);
 
-    double defaultEpsilon = logl_epsilon;
+//    double defaultEpsilon = logl_epsilon;
 
 	double begin_time = getRealTime();
 	double cur_lh;
@@ -947,7 +888,7 @@ double ModelFactory::optimizeParameters(bool fixed_len, bool write_info,
     // no optimization of branch length in the first round
     cur_lh = tree->computeLikelihood();
     tree->setCurScore(cur_lh);
-	if (verbose_mode >= VB_MED || write_info) 
+	if (write_info)
 		cout << "1. Initial log-likelihood: " << cur_lh << endl;
 
 	// For UpperBounds -----------
@@ -965,26 +906,42 @@ double ModelFactory::optimizeParameters(bool fixed_len, bool write_info,
         double new_lh;
 
         // changed to opimise edge length first, and then Q,W,R inside the loop by Thomas on Sept 11, 15
-		if (!fixed_len)
+		if (fixed_len == BRLEN_OPTIMIZE)
 			new_lh = tree->optimizeAllBranches(min(i,3), logl_epsilon);  // loop only 3 times in total (previously in v0.9.6 5 times)
+        else if (fixed_len == BRLEN_SCALE) {
+            double scaling = 1.0;
+            new_lh = tree->optimizeTreeLengthScaling(MIN_BRLEN_SCALE, scaling, MAX_BRLEN_SCALE, gradient_epsilon);
+        }
         new_lh = optimizeParametersOnly(gradient_epsilon);
 
 		if (new_lh == 0.0) {
-			if (!fixed_len) cur_lh = tree->optimizeAllBranches(100, logl_epsilon);
+            if (fixed_len == BRLEN_OPTIMIZE)
+                cur_lh = tree->optimizeAllBranches(100, logl_epsilon);
+            else if (fixed_len == BRLEN_SCALE) {
+                double scaling = 1.0;
+                cur_lh = tree->optimizeTreeLengthScaling(MIN_BRLEN_SCALE, scaling, MAX_BRLEN_SCALE, gradient_epsilon);
+            }
 			break;
 		}
 		if (verbose_mode >= VB_MED) {
 			model->writeInfo(cout);
 			site_rate->writeInfo(cout);
+            if (fixed_len == BRLEN_SCALE)
+                cout << "Scaled tree length: " << tree->treeLength() << endl;
 		}
 		if (new_lh > cur_lh + logl_epsilon) {
 			cur_lh = new_lh;
-			if (verbose_mode >= VB_MED || write_info)
+			if (write_info)
 				cout << i << ". Current log-likelihood: " << cur_lh << endl;
 		} else {
 			site_rate->classifyRates(new_lh);
-			if (!fixed_len) cur_lh = tree->optimizeAllBranches(100, logl_epsilon);
-				break;
+            if (fixed_len == BRLEN_OPTIMIZE)
+                cur_lh = tree->optimizeAllBranches(100, logl_epsilon);
+            else if (fixed_len == BRLEN_SCALE) {
+                double scaling = 1.0;
+                cur_lh = tree->optimizeTreeLengthScaling(MIN_BRLEN_SCALE, scaling, MAX_BRLEN_SCALE, gradient_epsilon);
+            }
+            break;
 		}
 	}
 
@@ -993,6 +950,8 @@ double ModelFactory::optimizeParameters(bool fixed_len, bool write_info,
     {
         double mean_rate = site_rate->rescaleRates();
         if (mean_rate != 1.0) {
+            if (fixed_len == BRLEN_FIX)
+                outError("Fixing branch lengths not supported under specified site rate model");
             tree->scaleLength(mean_rate);
             tree->clearAllPartialLH();
         }
@@ -1011,6 +970,8 @@ double ModelFactory::optimizeParameters(bool fixed_len, bool write_info,
 	if (verbose_mode <= VB_MIN && write_info) {
 		model->writeInfo(cout);
 		site_rate->writeInfo(cout);
+        if (fixed_len == BRLEN_SCALE)
+            cout << "Scaled tree length: " << tree->treeLength() << endl;
 	}
 	double elapsed_secs = getRealTime() - begin_time;
 	if (write_info)
