@@ -86,6 +86,12 @@ public:
 	 */
 	virtual int getNMixtures() {return size(); }
 
+ 	/**
+	 * @param cat mixture class
+	 * @return weight of a mixture model component
+	 */
+	virtual double getMixtureWeight(int cat) { return prop[cat]; }
+
 	/**
 		@return the number of dimensions
 	*/
@@ -123,6 +129,11 @@ public:
 	*/
 	virtual double optimizeParameters(double gradient_epsilon);
 
+	/**
+	 * @return TRUE if parameters are at the boundary that may cause numerical unstability
+	 */
+	virtual bool isUnstableParameters();
+
 	/**
 		decompose the rate matrix into eigenvalues and eigenvectors
 	*/
@@ -155,6 +166,17 @@ public:
 	 */
 	virtual string getNameParams();
 
+    /**
+     * compute the memory size for the model, can be large for site-specific models
+     * @return memory size required in bytes
+     */
+    virtual uint64_t getMemoryRequired() {
+    	uint64_t mem = ModelGTR::getMemoryRequired();
+    	for (iterator it = begin(); it != end(); it++)
+    		mem += (*it)->getMemoryRequired();
+    	return mem;
+    }
+
 	/**
 		rates of mixture components
 	*/
