@@ -65,7 +65,12 @@
 #include "timeutil.h"
 //#include <unistd.h>
 #include <stdlib.h>
-#include "vectorclass/vectorclass.h"
+#include "vectorclass/instrset.h"
+
+#include "MPIHelper.h"
+#ifdef _IQTREE_MPI
+#include <mpi.h>
+#endif
 
 #ifdef _OPENMP
 	#include <omp.h>
@@ -200,6 +205,9 @@ inline void separator(ostream &out, int type = 0) {
 void printCopyright(ostream &out) {
 #ifdef IQ_TREE
  	out << "IQ-TREE";
+    #ifdef _IQTREE_MPI
+    out << " MPI";
+    #endif
 	#ifdef _OPENMP
 	out << " multicore";
 	#endif
@@ -227,7 +235,7 @@ void printCopyright(ostream &out) {
 #endif
 
 #ifdef IQ_TREE
-	out << endl << "Copyright (c) 2011-2015 Nguyen Lam Tung, Olga Chernomor, Arndt von Haeseler and Bui Quang Minh." << endl << endl;
+	out << endl << "Copyright (c) 2011-2016 Nguyen Lam Tung, Olga Chernomor, Arndt von Haeseler and Bui Quang Minh." << endl << endl;
 #else
 	out << endl << "Copyright (c) 2006-2014 Olga Chernomor, Arndt von Haeseler and Bui Quang Minh." << endl << endl;
 #endif
@@ -1536,7 +1544,7 @@ void branchStats(Params &params){
 	
 	/***** Following added by BQM to print internal branch lengths */
 	NodeVector nodes1, nodes2;
-	mytree.getAllInnerBranches(nodes1, nodes2);
+	mytree.generateNNIBraches(nodes1, nodes2);
 	output = params.out_prefix;
 	output += ".inlen";
 	try {
@@ -1711,36 +1719,39 @@ public:
     outstreambuf* open( const char* name, ios::openmode mode = ios::out);
     outstreambuf* close();
     ~outstreambuf() { close(); }
+    streambuf *get_fout_buf() {
+        return fout_buf;
+    }
+    ofstream *get_fout() {
+        return &fout;
+    }
     
 protected:
 	ofstream fout;
 	streambuf *cout_buf;
-	streambuf *cerr_buf;
 	streambuf *fout_buf;
     virtual int     overflow( int c = EOF);
     virtual int     sync();
 };
 
-
 outstreambuf* outstreambuf::open( const char* name, ios::openmode mode) {
-    fout.open(name, mode);
-	if (!fout.is_open()) {
-		cout << "Could not open " << name << " for logging" << endl;
-		return NULL;
-	}
+    if (!(Params::getInstance().suppress_output_flags & OUT_LOG) && MPIHelper::getInstance().isMaster()) {
+        fout.open(name, mode);
+        if (!fout.is_open()) {
+            cout << "Could not open " << name << " for logging" << endl;
+            return NULL;
+        }
+        fout_buf = fout.rdbuf();
+    }
 	cout_buf = cout.rdbuf();
-	cerr_buf = cerr.rdbuf();
-	fout_buf = fout.rdbuf();
 	cout.rdbuf(this);
-	cerr.rdbuf(this);
     return this;
 }
 
 outstreambuf* outstreambuf::close() {
+    cout.rdbuf(cout_buf);
     if ( fout.is_open()) {
         sync();
-        cout.rdbuf(cout_buf);
-        cerr.rdbuf(cerr_buf);
 		fout.close();
         return this;
     }
@@ -1748,32 +1759,79 @@ outstreambuf* outstreambuf::close() {
 }
 
 int outstreambuf::overflow( int c) { // used for output buffer only
-	if (verbose_mode >= VB_MIN)
+	if ((verbose_mode >= VB_MIN && MPIHelper::getInstance().isMaster()) || verbose_mode >= VB_MED)
 		if (cout_buf->sputc(c) == EOF) return EOF;
+    if (Params::getInstance().suppress_output_flags & OUT_LOG)
+        return c;
+    if (!MPIHelper::getInstance().isMaster())
+        return c;
 	if (fout_buf->sputc(c) == EOF) return EOF;
 	return c;
 }
 
+
+
 int outstreambuf::sync() { // used for output buffer only
-	if (verbose_mode >= VB_MIN)
+	if ((verbose_mode >= VB_MIN && MPIHelper::getInstance().isMaster()) || verbose_mode >= VB_MED)
 		cout_buf->pubsync();
+    if ((Params::getInstance().suppress_output_flags & OUT_LOG) || !MPIHelper::getInstance().isMaster())
+        return 0;        
 	return fout_buf->pubsync();
 }
 
+class errstreambuf : public streambuf {
+public:
+    void init(streambuf *fout_buf) {
+        this->fout_buf = fout_buf;
+        cerr_buf = cerr.rdbuf();
+        cerr.rdbuf(this);
+    }
+    
+    ~errstreambuf() {
+        cerr.rdbuf(cerr_buf);
+    }
+    
+protected:
+	streambuf *cerr_buf;
+	streambuf *fout_buf;
+    
+    virtual int overflow( int c = EOF) {
+        if (cerr_buf->sputc(c) == EOF) return EOF;
+        if ((Params::getInstance().suppress_output_flags & OUT_LOG))
+            return c;
+        if (fout_buf->sputc(c) == EOF) return EOF;
+        return c;
+    }
+    
+    virtual int sync() {
+        cerr_buf->pubsync();
+        if (Params::getInstance().suppress_output_flags & OUT_LOG)
+            return 0;        
+        return fout_buf->pubsync();
+    }
+};
+
+
+
+/*********************************************************************************
+ * GLOBAL VARIABLES
+ *********************************************************************************/
 outstreambuf _out_buf;
+errstreambuf _err_buf;
 string _log_file;
 int _exit_wait_optn = FALSE;
 
-
 extern "C" void startLogFile(bool append_log) {
     if (append_log)
         _out_buf.open(_log_file.c_str(), ios::app);
     else
         _out_buf.open(_log_file.c_str());
+    _err_buf.init(_out_buf.get_fout_buf());
 }
 
 extern "C" void endLogFile() {
 	_out_buf.close();
+    
 }
 
 void funcExit(void) {
@@ -1783,7 +1841,7 @@ void funcExit(void) {
 		while (getchar() != '\n');
 	}
 	
-	endLogFile();
+    endLogFile();
 }
 
 extern "C" void funcAbort(int signal_number)
@@ -1797,16 +1855,19 @@ extern "C" void funcAbort(int signal_number)
 	print_stacktrace(cerr);
 #endif
 
-	cout << endl << "*** IQ-TREE CRASHES WITH SIGNAL ";
+	cerr << endl << "*** IQ-TREE CRASHES WITH SIGNAL ";
 	switch (signal_number) {
-		case SIGABRT: cout << "ABORTED"; break;
-		case SIGFPE:  cout << "ERRONEOUS NUMERIC"; break;
-		case SIGILL:  cout << "ILLEGAL INSTRUCTION"; break;
-		case SIGSEGV: cout << "SEGMENTATION FAULT"; break;
+		case SIGABRT: cerr << "ABORTED"; break;
+		case SIGFPE:  cerr << "ERRONEOUS NUMERIC"; break;
+		case SIGILL:  cerr << "ILLEGAL INSTRUCTION"; break;
+		case SIGSEGV: cerr << "SEGMENTATION FAULT"; break;
+#if !defined WIN32 && !defined _WIN32 && !defined __WIN32__
+		case SIGBUS: cerr << "BUS ERROR"; break;
+#endif
 	}
-    cout << endl;
-	cout << "*** For bug report please send to developers:" << endl << "***    Log file: " << _log_file;
-	cout << endl << "***    Alignment files (if possible)" << endl;
+    cerr << endl;
+	cerr << "*** For bug report please send to developers:" << endl << "***    Log file: " << _log_file;
+	cerr << endl << "***    Alignment files (if possible)" << endl;
 	funcExit();
 	signal(signal_number, SIG_DFL);
 }
@@ -2134,57 +2195,69 @@ Instruction set ID reported by vectorclass::instrset_detect
 */
 int instruction_set;
 
-int main(int argc, char *argv[])
-{
+int main(int argc, char *argv[]) {
+#ifdef _IQTREE_MPI
+	double time_initial, time_current;
+	int n_tasks, task_id;
+	if (MPI_Init(&argc, &argv) != MPI_SUCCESS) {
+		outError("MPI initialization failed!");
+	}
+	MPI_Comm_size(MPI_COMM_WORLD, &n_tasks);
+	MPI_Comm_rank(MPI_COMM_WORLD, &task_id);
+	MPIHelper::getInstance().setNumProcesses(n_tasks);
+	MPIHelper::getInstance().setProcessID(task_id);
+	MPIHelper::getInstance().setNumTreeReceived(0);
+	MPIHelper::getInstance().setNumTreeSent(0);
+    MPIHelper::getInstance().setNumNNISearch(0);
+#endif
 
 	/*************************/
 	{ /* local scope */
-		int found=FALSE;              /* "click" found in cmd name? */
+		int found = FALSE;              /* "click" found in cmd name? */
 		int n, dummyint;
 		char *tmpstr;
-		int     intargc; 
-		char  **intargv; 
-		intargc = 0; 
-		intargv = NULL; 
-		
-		for (n = strlen(argv[0]) - 5; 
-		    (n >= 0) && !found && (argv[0][n] != '/')
-		             && (argv[0][n] != '\\'); n--) {
+		int intargc;
+		char **intargv;
+		intargc = 0;
+		intargv = NULL;
+
+		for (n = strlen(argv[0]) - 5;
+			 (n >= 0) && !found && (argv[0][n] != '/')
+			 && (argv[0][n] != '\\'); n--) {
 
 			tmpstr = &(argv[0][n]);
 			dummyint = 0;
-			(void)sscanf(tmpstr, "click%n", &dummyint);
+			(void) sscanf(tmpstr, "click%n", &dummyint);
 			if (dummyint == 5) found = TRUE;
 			else {
 				dummyint = 0;
-				(void)sscanf(tmpstr, "CLICK%n", &dummyint);
+				(void) sscanf(tmpstr, "CLICK%n", &dummyint);
 				if (dummyint == 5) found = TRUE;
 				else {
 					dummyint = 0;
-					(void)sscanf(tmpstr, "Click%n", &dummyint);
+					(void) sscanf(tmpstr, "Click%n", &dummyint);
 					if (dummyint == 5) found = TRUE;
 				}
 			}
 		}
-		if(found) _exit_wait_optn = TRUE;
+		if (found) _exit_wait_optn = TRUE;
 
 		if (_exit_wait_optn) { // get commandline parameters from keyboard
-			getintargv(&intargc, &intargv); 
+			getintargv(&intargc, &intargv);
 			fprintf(stdout, "\n\n");
-			if(intargc > 1) { // if there were option entered, use them as argc/argv
-				argc = intargc; 
-				argv = intargv; 
-			} 
+			if (intargc > 1) { // if there were option entered, use them as argc/argv
+				argc = intargc;
+				argv = intargv;
+			}
 		}
 	} /* local scope */
 	/*************************/
 
-	//Params params;
 	parseArg(argc, argv, Params::getInstance());
 
     // 2015-12-05
     Checkpoint *checkpoint = new Checkpoint;
-    string filename = (string)Params::getInstance().out_prefix + ".ckp.gz";
+    string filename = (string)Params::getInstance().out_prefix +".ckp.gz";
     checkpoint->setFileName(filename);
     
     bool append_log = false;
@@ -2210,28 +2283,53 @@ int main(int argc, char *argv[])
         }
     }
 
+    // after loading, workers are not allowed to write checkpoint anymore
+    if (MPIHelper::getInstance().isWorker())
+        checkpoint->setFileName("");
 
 	_log_file = Params::getInstance().out_prefix;
 	_log_file += ".log";
 	startLogFile(append_log);
+	time_t start_time;
 
     if (append_log) {
         cout << endl << "******************************************************"
              << endl << "CHECKPOINT: Resuming analysis from " << filename << endl << endl;
     }
+#ifdef _IQTREE_MPI
+	cout << "************************************************" << endl;
+	cout << "* START TREE SEARCH USING MPI WITH " << MPIHelper::getInstance().getNumProcesses() << " PROCESSES *" << endl;
+	cout << "************************************************" << endl;
+	unsigned int rndSeed;
+	if (MPIHelper::getInstance().isMaster()) {
+		rndSeed = Params::getInstance().ran_seed;
+		cout << "Random seed of master = " << rndSeed << endl;
+	}
+	// Broadcast random seed
+	MPI_Bcast(&rndSeed, 1, MPI_INT, PROC_MASTER, MPI_COMM_WORLD);
+	if (MPIHelper::getInstance().isWorker()) {
+//		Params::getInstance().ran_seed = rndSeed + task_id * 100000;
+		Params::getInstance().ran_seed = rndSeed;
+//		printf("Process %d: random_seed = %d\n", task_id, Params::getInstance().ran_seed);
+	}
+#endif
 
 	atexit(funcExit);
 	signal(SIGABRT, &funcAbort);
 	signal(SIGFPE, &funcAbort);
 	signal(SIGILL, &funcAbort);
 	signal(SIGSEGV, &funcAbort);
+#if !defined WIN32 && !defined _WIN32 && !defined __WIN32__
+	signal(SIGBUS, &funcAbort);
+#endif
 	printCopyright(cout);
+
 	/*
-	double x=1e-100;
-	double y=1e-101;
-	if (x > y) cout << "ok!" << endl;
-	else cout << "shit!" << endl;
-	*/
+    double x=1e-100;
+    double y=1e-101;
+    if (x > y) cout << "ok!" << endl;
+    else cout << "shit!" << endl;
+    */
 	//FILE *pfile = popen("hostname","r");
 	char hostname[100];
 #if defined WIN32 || defined _WIN32 || defined __WIN32__
@@ -2251,10 +2349,10 @@ int main(int argc, char *argv[])
 #endif
 	if (instruction_set < 3) outError("Your CPU does not support SSE3!");
 	bool has_fma3 = (instruction_set >= 7) && hasFMA3();
-	bool has_fma4 = (instruction_set >= 7) && hasFMA4();
+//	bool has_fma4 = (instruction_set >= 7) && hasFMA4();
 
 #ifdef __FMA__
-	bool has_fma =  has_fma3 || has_fma4;
+	bool has_fma =  has_fma3;
 	if (!has_fma) {
 		outError("Your CPU does not support FMA instruction, quiting now...");
 	}
@@ -2262,16 +2360,19 @@ int main(int argc, char *argv[])
 
 	cout << "Host:    " << hostname << " (";
 	switch (instruction_set) {
+	case 0: cout << "80386, "; break;
+	case 1: cout << "SSE, "; break;
+	case 2: cout << "SSE2, "; break;
 	case 3: cout << "SSE3, "; break;
 	case 4: cout << "SSSE3, "; break;
 	case 5: cout << "SSE4.1, "; break;
 	case 6: cout << "SSE4.2, "; break;
 	case 7: cout << "AVX, "; break;
 	case 8: cout << "AVX2, "; break;
-	default: cout << "AVX512F, "; break;
+	default: cout << "AVX512, "; break;
 	}
 	if (has_fma3) cout << "FMA3, ";
-	if (has_fma4) cout << "FMA4, ";
+//	if (has_fma4) cout << "FMA4, ";
 //#if defined __APPLE__ || defined __MACH__
 	cout << (int)(((getMemorySize()/1024.0)/1024)/1024) << " GB RAM)" << endl;
 //#else
@@ -2286,16 +2387,20 @@ int main(int argc, char *argv[])
 
     checkpoint->get("iqtree.seed", Params::getInstance().ran_seed);
 	cout << "Seed:    " << Params::getInstance().ran_seed <<  " ";
-	init_random(Params::getInstance().ran_seed, true);
+	init_random(Params::getInstance().ran_seed + MPIHelper::getInstance().getProcessID(), true);
 
-	time_t start_time;
 	time(&start_time);
 	cout << "Time:    " << ctime(&start_time);
 
-	if (Params::getInstance().lk_no_avx)
+	if (Params::getInstance().lk_no_avx == 1)
 		instruction_set = min(instruction_set, 6);
 
 	cout << "Kernel:  ";
+
+    if (Params::getInstance().lk_safe_scaling) {
+        cout << "Safe ";
+    }
+
 	if (Params::getInstance().pll) {
 #ifdef __AVX__
 		cout << "PLL-AVX";
@@ -2303,10 +2408,13 @@ int main(int argc, char *argv[])
 		cout << "PLL-SSE3";
 #endif
 	} else {
+        bool has_fma = (has_fma3) && (instruction_set >= 7) && Params::getInstance().lk_no_avx != 2;
 		switch (Params::getInstance().SSE) {
 		case LK_EIGEN: cout << "No SSE"; break;
 		case LK_EIGEN_SSE:
-			if (instruction_set >= 7) {
+            if (has_fma) {
+                cout << "AVX+FMA";
+            } else if (instruction_set >= 7) {
 				cout << "AVX";
 			} else {
 				cout << "SSE3";
@@ -2319,18 +2427,23 @@ int main(int argc, char *argv[])
 		}
 	}
 
-
-
 #ifdef _OPENMP
-	if (Params::getInstance().num_threads == 0) {
+	if (Params::getInstance().num_threads < 0) {
 		cout << endl << endl;
-		outError("Please specify the number of cores to use (-nt option)!");
+		outError("Please specify number of cores via -nt option. Use '-nt AUTO' to automatically determine the best number of cores");
 	}
-	if (Params::getInstance().num_threads) omp_set_num_threads(Params::getInstance().num_threads);
+	if (Params::getInstance().num_threads >= 1) {
+        omp_set_num_threads(Params::getInstance().num_threads);
+        Params::getInstance().num_threads = omp_get_max_threads();
+    }
 //	int max_threads = omp_get_max_threads();
-	Params::getInstance().num_threads = omp_get_max_threads();
 	int max_procs = countPhysicalCPUCores();
-	cout << " - " << Params::getInstance().num_threads  << " threads (" << max_procs << " CPU cores detected)";
+	cout << " - ";
+    if (Params::getInstance().num_threads > 0)
+        cout << Params::getInstance().num_threads  << " threads";
+    else
+        cout << "auto-detect";
+    cout << "(" << max_procs << " CPU cores detected)";
 	if (Params::getInstance().num_threads  > max_procs) {
 		cout << endl;
 		outError("You have specified more threads than CPU cores available");
@@ -2342,9 +2455,11 @@ int main(int argc, char *argv[])
 		outError("Number of threads must be 1 for sequential version.");
 	}
     int num_procs = countPhysicalCPUCores();
+#ifndef _IQTREE_MPI
     if (num_procs > 1) {
         cout << endl << endl << "NOTE: Consider using the multicore version because your CPU has " << num_procs << " cores!";
     }
+#endif
 #endif
 	//cout << "sizeof(int)=" << sizeof(int) << endl;
 	cout << endl << endl;
@@ -2390,6 +2505,13 @@ int main(int argc, char *argv[])
     CKP_SAVE(version);
     checkpoint->endStruct();
 
+    if (MPIHelper::getInstance().getNumProcesses() > 1) {
+        if (Params::getInstance().aln_file || Params::getInstance().partition_file) {
+            runPhyloAnalysis(Params::getInstance(), checkpoint);
+        } else {
+            outError("Please use one MPI process! The feature you wanted does not need parallelization.");
+        }
+    } else
 	// call the main function
 	if (Params::getInstance().tree_gen != NONE) {
 		generateRandomTree(Params::getInstance());
@@ -2485,10 +2607,14 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	delete checkpoint;
 	time(&start_time);
 	cout << "Date and Time: " << ctime(&start_time);
+	delete checkpoint;
 
 	finish_random();
+    
+#ifdef _IQTREE_MPI
+    MPI_Finalize();
+#endif    
 	return EXIT_SUCCESS;
 }
