@@ -96,7 +96,7 @@ public:
 	 */
 	virtual void initSettings(Params& params);
 
-    virtual void setLikelihoodKernel(LikelihoodKernel lk);
+    virtual void setLikelihoodKernel(LikelihoodKernel lk, int num_threads);
 
     virtual void changeLikelihoodKernel(LikelihoodKernel lk);
 
@@ -212,6 +212,11 @@ public:
      */
     virtual double computeLikelihood(double *pattern_lh = NULL);
 
+    /**
+     * @return number of elements per site lhl entry, used in conjunction with computePatternLhCat
+     */
+    virtual int getNumLhCat(SiteLoglType wsl);
+
     /**
             compute pattern likelihoods only if the accumulated scaling factor is non-zero.
             Otherwise, copy the pattern_lh attribute
@@ -223,6 +228,13 @@ public:
     virtual void computePatternLikelihood(double *pattern_lh, double *cur_logl = NULL,
     		double *pattern_lh_cat = NULL, SiteLoglType wsl = WSL_RATECAT);
 
+    /**
+            compute pattern posterior probabilities per rate/mixture category
+            @param pattern_prob_cat (OUT) all pattern-probabilities per category
+            @param wsl either WSL_RATECAT, WSL_MIXTURE or WSL_MIXTURE_RATECAT
+     */
+    virtual void computePatternProbabilityCategory(double *pattern_prob_cat, SiteLoglType wsl);
+
     /**
             optimize all branch lengths of all subtrees, then compute branch lengths
             of supertree as weighted average over all subtrees
@@ -253,6 +265,7 @@ public:
     virtual void changeNNIBrans(NNIMove nnimove);
 
     /**
+        OBSOLETE!
      * 	 Restore the branch lengths from the saved values
 	 * @param node the current node of the post-order tree traversal
 	 * @param dad the dad of that node used to direct the traversal
@@ -309,7 +322,7 @@ public:
      * compute the memory size required for storing partial likelihood vectors
      * @return memory size required in bytes
      */
-    virtual uint64_t getMemoryRequired(size_t ncategory = 1);
+    virtual uint64_t getMemoryRequired(size_t ncategory = 1, bool full_mem = false);
 
     /**
      * count the number of super branches that map to no branches in gene trees
@@ -326,7 +339,7 @@ public:
      */
     virtual int fixNegativeBranch(bool force = false, Node *node = NULL, Node *dad = NULL);
 
-    virtual int computeParsimonyBranch(PhyloNeighbor *dad_branch, PhyloNode *dad, int *branch_subst = NULL);
+    virtual int computeParsimonyBranchObsolete(PhyloNeighbor *dad_branch, PhyloNode *dad, int *branch_subst = NULL);
 
     /** True when mixed codon with other data type */
     bool rescale_codon_brlen;
