@@ -1,6 +1,8 @@
 /***************************************************************************
- *   Copyright (C) 2006 by BUI Quang Minh, Steffen Klaere, Arndt von Haeseler   *
- *   minh.bui@univie.ac.at   *
+ *   Copyright (C) 2009-2015 by                                            *
+ *   BUI Quang Minh <minh.bui@univie.ac.at>                                *
+ *   Lam-Tung Nguyen <nltung@gmail.com>                                    *
+ *                                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -32,6 +34,8 @@
 #include <stdlib.h>
 #include <math.h>
 #include <stdint.h>
+#include <string.h>
+#include <sstream>
 
 //#include <sys/time.h>
 //#include <time.h>
@@ -43,13 +47,28 @@
 #define SPRNG
 #include "sprng/sprng.h"
 
+// redefine assertion
+inline void _my_assert(const char* expression, const char *func, const char* file, int line)
+{
+    char *sfile = (char*)strrchr(file, '/');
+    if (!sfile) sfile = (char*)file; else sfile++;
+    cerr << "Assertion (" << expression << ") failed, function " << func << ", file " << sfile << ", line " << line << endl;
+    abort();
+}
+ 
+#ifdef NDEBUG
+#define ASSERT(EXPRESSION) ((void)0)
+#else
+#define ASSERT(EXPRESSION) ((EXPRESSION) ? (void)0 : _my_assert(#EXPRESSION, __func__, __FILE__, __LINE__))
+#endif
+
 
 #define USE_HASH_MAP
 
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(GCC_VERSION)
 #define GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
-#else
-#define GCC_VERSION 0
+//#else
+//#define GCC_VERSION 0
 #endif
 
 // for MSVC
@@ -58,13 +77,23 @@
 #endif
 
 #if defined(USE_HASH_MAP)
+//    #include <unordered_map>
+//    #include <unordered_set>
+
 	#if defined(_MSC_VER)
 		#include <unordered_map>
 		#include <unordered_set>
     #elif defined(__clang__)
-		#include <tr1/unordered_map>
-		#include <tr1/unordered_set>
-		using namespace std::tr1;    
+        // libc++ detected:     _LIBCPP_VERSION
+        // libstdc++ detected:  __GLIBCXX__
+        #if __has_include(<unordered_map>) // defines _LIBCPP_VERSION
+            #include <unordered_map>
+            #include <unordered_set>
+        #else
+            #include <tr1/unordered_map>
+            #include <tr1/unordered_set>
+            using namespace std::tr1;    
+        #endif
 	#elif !defined(__GNUC__)
 		#include <hash_map>
 		#include <hash_set>
@@ -80,6 +109,7 @@
 		#include <tr1/unordered_set>
 		using namespace std::tr1;
 	#endif
+
 #else
 	#include <map>
 	#include <set>
@@ -207,7 +237,7 @@ typedef vector<string> StrVector;
 /**
         matrix of double number
  */
-#define matrix(T) vector<vector<T> >
+#define matrix(T) vector< vector<T> >
 
 /**
         matrix of double
@@ -272,6 +302,7 @@ const int WT_NEWLINE = 128;
 const int WT_BR_LEN_FIXED_WIDTH = 256;
 const int WT_BR_ID = 512;
 const int WT_BR_LEN_ROUNDING = 1024;
+const int WT_BR_LEN_SHORT = 2048; // store only 6 digits after the comma for branch lengths
 const int TRUE = 1;
 const int FALSE = 0;
 
@@ -399,13 +430,38 @@ enum LikelihoodKernel {
 };
 
 enum LhMemSave {
-	LM_DETECT, LM_ALL_BRANCH, LM_PER_NODE
+	LM_PER_NODE, LM_MEM_SAVE
 };
 
 enum SiteLoglType {
     WSL_NONE, WSL_SITE, WSL_RATECAT, WSL_MIXTURE, WSL_MIXTURE_RATECAT
 };
 
+enum SiteFreqType {
+    WSF_NONE, WSF_POSTERIOR_MEAN, WSF_POSTERIOR_MAX
+};
+
+enum AncestralSeqType {
+    AST_NONE, AST_MARGINAL, AST_JOINT
+};
+
+const int BRLEN_OPTIMIZE = 0; // optimize branch lengths
+const int BRLEN_FIX      = 1; // fix branch lengths
+const int BRLEN_SCALE    = 2; // scale branch lengths
+
+const int OUT_LOG       = 1; // .log file written or not
+const int OUT_TREEFILE  = 2; // .treefile file written or not
+const int OUT_IQTREE    = 4; // .iqtree file written or not
+const int OUT_UNIQUESEQ = 8; // .uniqueseq file written or not
+
+
+const double MIN_GAMMA_RATE = 1e-6;
+// change from 0.01 to 0.02 as 0.01 causes numerical problems
+const double MIN_GAMMA_SHAPE = 0.02;
+const double MAX_GAMMA_SHAPE = 1000.0;
+const double TOL_GAMMA_SHAPE = 0.001;
+
+
 /** maximum number of newton-raphson steps for NNI branch evaluation */
 extern int NNI_MAX_NR_STEP;
 
@@ -428,6 +484,32 @@ private:
     //void operator=(Params const&) {}; // Disable assignment
 public:
 
+    /**
+    *  Fast and accurate optimiation for alpha and p_invar
+    */
+    bool fai;
+
+    /**
+     *  Option to check memory consumption only
+     */
+    bool memCheck;
+
+    /**
+     *  The support threshold for stable splits (Default = 0.9)
+     */
+    double stableSplitThreshold;
+
+    /**
+     *  Option for adaptive perturbation.
+     *  Branches that are shared among all candidate trees will be perturbed
+     */
+    bool adaptPertubation;
+
+	/**
+	 *  Option to do mutlipe start for estimating alpha and p_invar
+	 */
+	bool testAlpha;
+
     /**
      *  Restart the optimization of alpha and pinvar from different starting
      *  pinv values (supercedes the option testAlpha
@@ -464,17 +546,30 @@ public:
     bool exh_ai;
 
 	/**
-	 *  User file contains the alpha and invar parameters
+	 *  Text file contain all pairs of alpha and p_invar to
+	 *  evaluate.
+	 *  TODO Remove this option and implement the exhaustive search
+	 *  directly into IQ-TREE
 	 */
 	char* alpha_invar_file;
 
 	/**
-	 * Turn on feature to identify stable splits and fix them during tree search
+	 *  Enable tabu search for NNI
 	 */
-	bool fix_stable_splits;
+	bool tabu;
+
+    /**
+	 *  Use (5+5)-ES strategy
+	 */
+	bool five_plus_five;
 
 	/**
-	 *  Number of distinct locally optimal trees
+	 * Turn on feature to identify stable splits and fix them during tree search
+	 */
+	bool fixStableSplits;
+
+	/**
+	 *  Number of best trees used to compute stable splits
 	 */
 	int numSupportTrees;
 
@@ -496,11 +591,14 @@ public:
 
 	/**
 	 *  Number of best trees in the candidate set used to generate perturbed trees
+	 *  In term of evolutionary algorithm, this is the population size
 	 */
 	int popSize;
 
 	/**
 	 *  Maximum number of trees stored in the candidate tree set
+	 *  This is just a technical constraint to ensure that the candidate tree set
+	 *  does not have to store a lot of trees
 	 */
 	int maxCandidates;
 
@@ -509,10 +607,6 @@ public:
 	 */
 	bool speednni;
 
-	/**
-	 *  use reduction technique to constraint tree space
-	 */
-	bool reduction;
 
 	/**
 	 *  portion of NNI used for perturbing the tree
@@ -522,7 +616,7 @@ public:
 	/**
 	 *  logl epsilon for model parameter optimization
 	 */
-	double modeps;
+	double modelEps;
 
 	/**
 	 *  New search heuristics (DEFAULT: ON)
@@ -645,6 +739,9 @@ public:
     /* type of starting tree */
     START_TREE_TYPE start_tree;
 
+    /** name of constraint tree file in NEWICK format */
+    char *constraint_tree_file;
+
     /**
             prefix of the output file, default is the same as input file
      */
@@ -1199,6 +1296,11 @@ public:
      */
     double gamma_shape;
 
+    /**
+            minimum shape parameter (alpha) of the Gamma distribution for site rates
+     */
+    double min_gamma_shape;
+
     /**
             TRUE to use median rate for discrete categories, FALSE to use mean rate instead
      */
@@ -1226,9 +1328,11 @@ public:
     string optimize_alg_gammai;
 
     /**
-            TRUE if you want to fix branch lengths during model optimization
+            BRLEN_OPTIMIZE optimize branch lengths during model optimization
+            BRLEN_FIX      fix branch lengths during model optimization
+            BRLEN_SCALE    scale all branch lengths by the same factor during model optimization
      */
-    bool fixed_branch_length;
+    int fixed_branch_length;
 
     /** minimum branch length for optimization, default 0.000001 */
     double min_branch_length;
@@ -1270,15 +1374,23 @@ public:
     char *bootstrap_spec;
 
     /**
-            1 if output all intermediate trees from every IQPNNI iteration
+            1 if output all intermediate trees (initial trees, NNI-optimal trees and trees after each NNI step)
             2 if output all intermediate trees + 1-NNI-away trees
      */
     int write_intermediate_trees;
 
     /**
-     *  Write out all candidate trees (the locally optimal trees)
+     *  Write all distinct intermediate trees and there likelihoods
+     *  Note: intermediate trees are trees that have been visited by the search. These include trees created by
+     *  NNI-steps within each NNI iteration.
      */
-    int write_local_optimal_trees;
+    bool writeDistImdTrees;
+
+    /**
+     *  Write trees obtained at the end of each NNI search
+     */
+    bool write_candidate_trees;
+
 
     /**
         TRUE to avoid duplicated trees while writing intermediate trees
@@ -1339,7 +1451,13 @@ public:
     LikelihoodKernel SSE;
 
     /** TRUE to not use AVX even available in CPU, default: FALSE */
-    bool lk_no_avx;
+    int lk_no_avx;
+
+    /** TRUE for safe numerical scaling (per category; used for large trees), default: FALSE */
+    bool lk_safe_scaling;
+
+    /** minimum number of sequences to always use safe scaling, default: 2000 */
+    int numseq_safe_scaling;
 
     /**
      	 	WSL_NONE: do not print anything
@@ -1347,20 +1465,43 @@ public:
             WSL_RATECAT: print site log-likelihood per rate category
             WSL_MIXTURE: print site log-likelihood per mixture class
             WSL_MIXTURE_RATECAT: print site log-likelihood per mixture class per rate category
+            WSL_STATE: print site log-likelihood per state
      */
     SiteLoglType print_site_lh;
 
+    /** TRUE to print partition log-likelihood, default: FALSE */
+    bool print_partition_lh;
+
+    /**
+        control printing posterior probability of each site belonging to a rate/mixture categories
+        same meaning as print_site_lh, but results are printed to .siteprob file
+        WSL_RATECAT: print site probability per rate category
+        WSL_MIXTURE: print site probability per mixture class
+        WSL_MIXTURE_RATECAT: print site probability per mixture class per rate category
+    */
+    SiteLoglType print_site_prob;
+
+    /**
+        AST_NONE: do not print ancestral sequences (default)
+        AST_MARGINAL: print ancestral sequences by marginal reconstruction
+        AST_JOINT: print ancestral sequences by joint reconstruction
+    */
+    AncestralSeqType print_ancestral_sequence;
+
+    /** minimum probability to assign an ancestral state */
+    double min_ancestral_prob;
+
     /**
         0: print nothing
         1: print site state frequency vectors
     */
-    int print_site_state_freq;
+    SiteFreqType print_site_state_freq;
 
     /** TRUE to print site-specific rates, default: FALSE */
     bool print_site_rate;
 
-    /* 1: print site posterior probability */
-    int print_site_posterior;
+    /* 1: print site posterior probability for many trees during tree search */
+    int print_trees_site_posterior;
 
     /**
             TRUE to print tree log-likelihood
@@ -1582,6 +1723,8 @@ public:
 	/** true to print all UFBoot trees to a file */
 	int print_ufboot_trees;
 
+    int contree_rfdist;
+
     /****** variables for NNI cutoff heuristics ******/
 
     /**
@@ -1639,6 +1782,11 @@ public:
      */
     char *site_freq_file;
 
+    /**
+        user tree file used to estimate site-specific state frequency model 
+    */
+    char *tree_freq_file;
+
     /** number of threads for OpenMP version     */
     int num_threads;
 
@@ -1679,6 +1827,9 @@ public:
 	 * 1: only store 1 partial likelihood vector per node */
 	LhMemSave lh_mem_save;
 
+    /** maximum size of memory allowed to use */
+    double max_mem_size;
+
 	/* TRUE to print .splits file in star-dot format */
 	bool print_splits_file;
     
@@ -1690,7 +1841,6 @@ public:
 
     /** frequencies of const patterns to be inserted into alignment */
     char *freq_const_patterns;
-
     /** BQM 2015-02-25: true to NOT rescale Gamma+Invar rates by (1-p_invar) */
     bool no_rescale_gamma_invar;
 
@@ -1715,6 +1865,13 @@ public:
     /** true if ignoring the "finished" flag in checkpoint file */
     bool force_unfinished;
 
+    /** control output files to be written
+     * OUT_LOG
+     * OUT_TREEFILE
+     * OUT_IQTREE
+     */
+    int suppress_output_flags;
+
 };
 
 /**
@@ -1863,8 +2020,7 @@ const char ERR_WRITE_OUTPUT[] = "Cannot write to file ";
 const char ERR_NO_K[] = "You must specify the number of taxa in the PD set.";
 const char ERR_TOO_SMALL_K[] = "Size of PD-set must be at least the size of initial set.";
 const char ERR_NO_BUDGET[] = "Total budget is not specified or less than zero.";
-const char ERR_TOO_SMALL_BUDGET[] = "Not enough budget to conserve the inital set of taxa.";
-
+const char ERR_TOO_SMALL_BUDGET[] = "Not enough budget to conserve the initial set of taxa.";
 const char ERR_INTERNAL[] = "Internal error, pls contact authors!";
 
 /*--------------------------------------------------------------*/
@@ -1930,7 +2086,7 @@ int64_t convert_int64(const char *str) throw (string);
         @param end_pos end position
         @return the number
  */
-int64_t convert_int64(const char *str, int64_t &end_pos) throw (string);
+int64_t convert_int64(const char *str, int &end_pos) throw (string);
 
 /**
         convert string to double, with error checking
@@ -2351,4 +2507,29 @@ inline uint32_t popcount_lauradoux(unsigned *buf, int n) {
  */
 bool memcmpcpy(void * destination, const void * source, size_t num);
 
+/**
+ *  Generating a unique integer from a pair of 2 integer
+ *  This method is called cantor pairing function (see wikepedia).
+ *  @param int1 the first integer
+ *  @param int2 the second integer
+ *  @return the encoding of the 2 integer
+ */
+int pairInteger(int int1, int int2);
+
+template <typename T>
+string NumberToString ( T Number )
+{
+    ostringstream ss;
+    ss << Number;
+    return ss.str();
+}
+
+template <typename T>
+T StringToNumber ( const string &Text )
+{
+    istringstream ss(Text);
+    T result;
+    return ss >> result ? result : 0;
+}
+
 #endif
